@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # flake8: NOQA W605
 """Helper methods and mixins.
 
@@ -6,28 +5,33 @@ libtmux.common
 ~~~~~~~~~~~~~~
 
 """
-import collections
 import logging
 import os
 import re
 import subprocess
 import sys
+import typing as t
+from collections.abc import MutableMapping
 from distutils.version import LooseVersion
 
 from . import exc
-from ._compat import MutableMapping, console_to_str, str_from_console
+from ._compat import console_to_str, str_from_console
 
 logger = logging.getLogger(__name__)
 
 
 #: Minimum version of tmux required to run libtmux
-TMUX_MIN_VERSION = '1.8'
+TMUX_MIN_VERSION = "1.8"
 
 #: Most recent version of tmux supported
-TMUX_MAX_VERSION = '2.4'
+TMUX_MAX_VERSION = "2.4"
+
+SessionDict = t.Dict[str, t.Any]
+WindowDict = t.Dict[str, t.Any]
+PaneDict = t.Dict[str, t.Any]
 
 
-class EnvironmentMixin(object):
+class EnvironmentMixin:
 
     """
     Mixin class for managing session and server level environment variables in
@@ -50,7 +54,7 @@ class EnvironmentMixin(object):
         option : str
             environment value.
         """
-        args = ['set-environment']
+        args = ["set-environment"]
         if self._add_option:
             args += [self._add_option]
 
@@ -59,9 +63,9 @@ class EnvironmentMixin(object):
         proc = self.cmd(*args)
 
         if proc.stderr:
-            if isinstance(proc.stderr, list) and len(proc.stderr) == int(1):
+            if isinstance(proc.stderr, list) and len(proc.stderr) == 1:
                 proc.stderr = proc.stderr[0]
-            raise ValueError('tmux set-environment stderr: %s' % proc.stderr)
+            raise ValueError("tmux set-environment stderr: %s" % proc.stderr)
 
     def unset_environment(self, name):
         """
@@ -72,17 +76,17 @@ class EnvironmentMixin(object):
         name : str
             the environment variable name. such as 'PATH'.
         """
-        args = ['set-environment']
+        args = ["set-environment"]
         if self._add_option:
             args += [self._add_option]
-        args += ['-u', name]
+        args += ["-u", name]
 
         proc = self.cmd(*args)
 
         if proc.stderr:
-            if isinstance(proc.stderr, list) and len(proc.stderr) == int(1):
+            if isinstance(proc.stderr, list) and len(proc.stderr) == 1:
                 proc.stderr = proc.stderr[0]
-            raise ValueError('tmux set-environment stderr: %s' % proc.stderr)
+            raise ValueError("tmux set-environment stderr: %s" % proc.stderr)
 
     def remove_environment(self, name):
         """Remove environment variable ``$ tmux set-environment -r <name>``.
@@ -92,17 +96,17 @@ class EnvironmentMixin(object):
         name : str
             the environment variable name. such as 'PATH'.
         """
-        args = ['set-environment']
+        args = ["set-environment"]
         if self._add_option:
             args += [self._add_option]
-        args += ['-r', name]
+        args += ["-r", name]
 
         proc = self.cmd(*args)
 
         if proc.stderr:
-            if isinstance(proc.stderr, list) and len(proc.stderr) == int(1):
+            if isinstance(proc.stderr, list) and len(proc.stderr) == 1:
                 proc.stderr = proc.stderr[0]
-            raise ValueError('tmux set-environment stderr: %s' % proc.stderr)
+            raise ValueError("tmux set-environment stderr: %s" % proc.stderr)
 
     def show_environment(self, name=None):
         """Show environment ``$ tmux show-environment -t [session] <name>``.
@@ -121,13 +125,13 @@ class EnvironmentMixin(object):
             environmental variables in dict, if no name, or str if name
             entered.
         """
-        tmux_args = ['show-environment']
+        tmux_args = ["show-environment"]
         if self._add_option:
             tmux_args += [self._add_option]
         if name:
             tmux_args += [name]
         vars = self.cmd(*tmux_args).stdout
-        vars = [tuple(item.split('=', 1)) for item in vars]
+        vars = [tuple(item.split("=", 1)) for item in vars]
         vars_dict = {}
         for t in vars:
             if len(t) == 2:
@@ -135,7 +139,7 @@ class EnvironmentMixin(object):
             elif len(t) == 1:
                 vars_dict[t[0]] = True
             else:
-                raise ValueError('unexpected variable %s', t)
+                raise ValueError("unexpected variable %s", t)
 
         if name:
             return vars_dict.get(name)
@@ -143,7 +147,7 @@ class EnvironmentMixin(object):
         return vars_dict
 
 
-class tmux_cmd(object):
+class tmux_cmd:
 
     """
     :term:`tmux(1)` command via :py:mod:`subprocess`.
@@ -185,12 +189,12 @@ class tmux_cmd(object):
 
     def __init__(self, *args, **kwargs):
         tmux_bin = which(
-            'tmux',
+            "tmux",
             default_paths=kwargs.get(
-                'tmux_search_paths',
-                ['/bin', '/sbin', '/usr/bin', '/usr/sbin', '/usr/local/bin'],
+                "tmux_search_paths",
+                ["/bin", "/sbin", "/usr/bin", "/usr/sbin", "/usr/local/bin"],
             ),
-            append_env_path=kwargs.get('append_env_path', True),
+            append_env_path=kwargs.get("append_env_path", True),
         )
         if not tmux_bin:
             raise (exc.TmuxCommandNotFound)
@@ -208,23 +212,23 @@ class tmux_cmd(object):
             stdout, stderr = self.process.communicate()
             returncode = self.process.returncode
         except Exception as e:
-            logger.error('Exception for %s: \n%s' % (subprocess.list2cmdline(cmd), e))
+            logger.error(f"Exception for {subprocess.list2cmdline(cmd)}: \n{e}")
 
         self.returncode = returncode
 
         self.stdout = console_to_str(stdout)
-        self.stdout = self.stdout.split('\n')
+        self.stdout = self.stdout.split("\n")
         self.stdout = list(filter(None, self.stdout))  # filter empty values
 
         self.stderr = console_to_str(stderr)
-        self.stderr = self.stderr.split('\n')
+        self.stderr = self.stderr.split("\n")
         self.stderr = list(filter(None, self.stderr))  # filter empty values
 
-        if 'has-session' in cmd and len(self.stderr):
+        if "has-session" in cmd and len(self.stderr):
             if not self.stdout:
                 self.stdout = self.stderr[0]
 
-        logger.debug('self.stdout for %s: \n%s' % (' '.join(cmd), self.stdout))
+        logger.debug("self.stdout for {}: \n{}".format(" ".join(cmd), self.stdout))
 
 
 class TmuxMappingObject(MutableMapping):
@@ -273,10 +277,10 @@ class TmuxMappingObject(MutableMapping):
         try:
             return self._info[self.formatter_prefix + key]
         except KeyError:
-            raise AttributeError('%s has no property %s' % (self.__class__, key))
+            raise AttributeError(f"{self.__class__} has no property {key}")
 
 
-class TmuxRelationalObject(object):
+class TmuxRelationalObject:
 
     """Base Class for managing tmux object child entities.  .. # NOQA
 
@@ -333,19 +337,20 @@ class TmuxRelationalObject(object):
         """
 
         # from https://github.com/serkanyersen/underscore.py
-        def by(val, *args):
-            for key, value in attrs.items():
+        def by(val) -> bool:
+            for key in attrs.keys():
                 try:
                     if attrs[key] != val[key]:
                         return False
                 except KeyError:
                     return False
-                return True
+            return True
 
+        # TODO add type hint
+        target_children = list(filter(by, self.children))
         if first:
-            return list(filter(by, self.children))[0]
-        else:
-            return list(filter(by, self.children))
+            return target_children[0]
+        return target_children
 
     def get_by_id(self, id):
         """
@@ -376,10 +381,16 @@ class TmuxRelationalObject(object):
 
 
 def which(
-    exe=None,
-    default_paths=['/bin', '/sbin', '/usr/bin', '/usr/sbin', '/usr/local/bin'],
-    append_env_path=True,
-):
+    exe: str,
+    default_paths: t.List[str] = [
+        "/bin",
+        "/sbin",
+        "/usr/bin",
+        "/usr/sbin",
+        "/usr/local/bin",
+    ],
+    append_env_path: bool = True,
+) -> t.Optional[str]:
     """
     Return path of bin. Python clone of /usr/bin/which.
 
@@ -403,7 +414,7 @@ def which(
     from salt.util - https://www.github.com/saltstack/salt - license apache
     """
 
-    def _is_executable_file_or_link(exe):
+    def _is_executable_file_or_link(exe: str) -> bool:
         # check for os.X_OK doesn't suffice because directory may executable
         return os.access(exe, os.X_OK) and (os.path.isfile(exe) or os.path.islink(exe))
 
@@ -416,7 +427,7 @@ def which(
     # win' for cases to find optional alternatives
     if append_env_path:
         search_path = (
-            os.environ.get('PATH') and os.environ['PATH'].split(os.pathsep) or list()
+            os.environ.get("PATH") and os.environ["PATH"].split(os.pathsep) or list()
         )
     else:
         search_path = []
@@ -429,14 +440,14 @@ def which(
         if _is_executable_file_or_link(full_path):
             return full_path
     logger.info(
-        '\'{0}\' could not be found in the following search path: '
-        '\'{1}\''.format(exe, search_path)
+        "'{}' could not be found in the following search path: "
+        "'{}'".format(exe, search_path)
     )
 
     return None
 
 
-def get_version():
+def get_version() -> LooseVersion:
     """
     Return tmux version.
 
@@ -451,29 +462,29 @@ def get_version():
     :class:`distutils.version.LooseVersion`
         tmux version according to :func:`libtmux.common.which`'s tmux
     """
-    proc = tmux_cmd('-V')
+    proc = tmux_cmd("-V")
     if proc.stderr:
-        if proc.stderr[0] == 'tmux: unknown option -- V':
+        if proc.stderr[0] == "tmux: unknown option -- V":
             if sys.platform.startswith("openbsd"):  # openbsd has no tmux -V
-                return LooseVersion('%s-openbsd' % TMUX_MAX_VERSION)
+                return LooseVersion("%s-openbsd" % TMUX_MAX_VERSION)
             raise exc.LibTmuxException(
-                'libtmux supports tmux %s and greater. This system'
-                ' is running tmux 1.3 or earlier.' % TMUX_MIN_VERSION
+                "libtmux supports tmux %s and greater. This system"
+                " is running tmux 1.3 or earlier." % TMUX_MIN_VERSION
             )
         raise exc.VersionTooLow(proc.stderr)
 
-    version = proc.stdout[0].split('tmux ')[1]
+    version = proc.stdout[0].split("tmux ")[1]
 
     # Allow latest tmux HEAD
-    if version == 'master':
-        return LooseVersion('%s-master' % TMUX_MAX_VERSION)
+    if version == "master":
+        return LooseVersion("%s-master" % TMUX_MAX_VERSION)
 
-    version = re.sub(r'[a-z-]', '', version)
+    version = re.sub(r"[a-z-]", "", version)
 
     return LooseVersion(version)
 
 
-def has_version(version):
+def has_version(version: str) -> bool:
     """
     Return affirmative if tmux version installed.
 
@@ -490,7 +501,7 @@ def has_version(version):
     return get_version() == LooseVersion(version)
 
 
-def has_gt_version(min_version):
+def has_gt_version(min_version: str) -> bool:
     """
     Return affirmative if tmux version greater than minimum.
 
@@ -507,7 +518,7 @@ def has_gt_version(min_version):
     return get_version() > LooseVersion(min_version)
 
 
-def has_gte_version(min_version):
+def has_gte_version(min_version: str) -> bool:
     """
     Return True if tmux version greater or equal to minimum.
 
@@ -524,7 +535,7 @@ def has_gte_version(min_version):
     return get_version() >= LooseVersion(min_version)
 
 
-def has_lte_version(max_version):
+def has_lte_version(max_version: str) -> bool:
     """
     Return True if tmux version less or equal to minimum.
 
@@ -541,7 +552,7 @@ def has_lte_version(max_version):
     return get_version() <= LooseVersion(max_version)
 
 
-def has_lt_version(max_version):
+def has_lt_version(max_version: str) -> bool:
     """
     Return True if tmux version less than minimum.
 
@@ -558,7 +569,7 @@ def has_lt_version(max_version):
     return get_version() < LooseVersion(max_version)
 
 
-def has_minimum_version(raises=True):
+def has_minimum_version(raises: bool = True) -> bool:
     """
     Return if tmux meets version requirement. Version >1.8 or above.
 
@@ -591,8 +602,8 @@ def has_minimum_version(raises=True):
     if get_version() < LooseVersion(TMUX_MIN_VERSION):
         if raises:
             raise exc.VersionTooLow(
-                'libtmux only supports tmux %s and greater. This system'
-                ' has %s installed. Upgrade your tmux to use libtmux.'
+                "libtmux only supports tmux %s and greater. This system"
+                " has %s installed. Upgrade your tmux to use libtmux."
                 % (TMUX_MIN_VERSION, get_version())
             )
         else:
@@ -600,7 +611,7 @@ def has_minimum_version(raises=True):
     return True
 
 
-def session_check_name(session_name):
+def session_check_name(session_name: str):
     """
     Raises exception session name invalid, modeled after tmux function.
 
@@ -619,17 +630,17 @@ def session_check_name(session_name):
     """
     if not session_name or len(session_name) == 0:
         raise exc.BadSessionName("tmux session names may not be empty.")
-    elif '.' in session_name:
+    elif "." in session_name:
         raise exc.BadSessionName(
-            "tmux session name \"%s\" may not contain periods.", session_name
+            'tmux session name "%s" may not contain periods.', session_name
         )
-    elif ':' in session_name:
+    elif ":" in session_name:
         raise exc.BadSessionName(
-            "tmux session name \"%s\" may not contain colons.", session_name
+            'tmux session name "%s" may not contain colons.', session_name
         )
 
 
-def handle_option_error(error):
+def handle_option_error(error: str):
     """Raises exception if error in option command found.
 
     In tmux 3.0, show-option and show-window-otion return invalid option instead of
@@ -656,17 +667,17 @@ def handle_option_error(error):
     :exc:`exc.OptionError`, :exc:`exc.UnknownOption`, :exc:`exc.InvalidOption`,
     :exc:`exc.AmbiguousOption`
     """
-    if 'unknown option' in error:
+    if "unknown option" in error:
         raise exc.UnknownOption(error)
-    elif 'invalid option' in error:
+    elif "invalid option" in error:
         raise exc.InvalidOption(error)
-    elif 'ambiguous option' in error:
+    elif "ambiguous option" in error:
         raise exc.AmbiguousOption(error)
     else:
         raise exc.OptionError(error)  # Raise generic option error
 
 
-def get_libtmux_version():
+def get_libtmux_version() -> LooseVersion:
     """Return libtmux version is a PEP386 compliant format.
 
     Returns
