@@ -6,11 +6,15 @@ libtmux.server
 """
 import logging
 import os
+import typing as t
 
 from . import exc, formats
 from .common import (
     EnvironmentMixin,
+    PaneDict,
+    SessionDict,
     TmuxRelationalObject,
+    WindowDict,
     has_gte_version,
     session_check_name,
     tmux_cmd,
@@ -65,9 +69,9 @@ class Server(TmuxRelationalObject, EnvironmentMixin):
     #: ``-2`` or ``-8``
     colors = None
     #: unique child ID used by :class:`~libtmux.common.TmuxRelationalObject`
-    child_id_attribute = 'session_id'
+    child_id_attribute = "session_id"
     #: namespace used :class:`~libtmux.common.TmuxMappingObject`
-    formatter_prefix = 'server_'
+    formatter_prefix = "server_"
 
     def __init__(
         self,
@@ -75,9 +79,9 @@ class Server(TmuxRelationalObject, EnvironmentMixin):
         socket_path=None,
         config_file=None,
         colors=None,
-        **kwargs
+        **kwargs,
     ):
-        EnvironmentMixin.__init__(self, '-g')
+        EnvironmentMixin.__init__(self, "-g")
         self._windows = []
         self._panes = []
 
@@ -110,22 +114,22 @@ class Server(TmuxRelationalObject, EnvironmentMixin):
 
         args = list(args)
         if self.socket_name:
-            args.insert(0, '-L{}'.format(self.socket_name))
+            args.insert(0, f"-L{self.socket_name}")
         if self.socket_path:
-            args.insert(0, '-S{}'.format(self.socket_path))
+            args.insert(0, f"-S{self.socket_path}")
         if self.config_file:
-            args.insert(0, '-f{}'.format(self.config_file))
+            args.insert(0, f"-f{self.config_file}")
         if self.colors:
             if self.colors == 256:
-                args.insert(0, '-2')
+                args.insert(0, "-2")
             elif self.colors == 88:
-                args.insert(0, '-8')
+                args.insert(0, "-8")
             else:
-                raise ValueError('Server.colors must equal 88 or 256')
+                raise ValueError("Server.colors must equal 88 or 256")
 
         return tmux_cmd(*args, **kwargs)
 
-    def _list_sessions(self):
+    def _list_sessions(self) -> t.List[SessionDict]:
         """
         Return list of sessions in :py:obj:`dict` form.
 
@@ -140,36 +144,37 @@ class Server(TmuxRelationalObject, EnvironmentMixin):
         """
 
         sformats = formats.SESSION_FORMATS
-        tmux_formats = ['#{%s}' % f for f in sformats]
+        tmux_formats = ["#{%s}" % f for f in sformats]
 
-        tmux_args = ('-F%s' % '\t'.join(tmux_formats),)  # output
+        tmux_args = ("-F%s" % formats.FORMAT_SEPARATOR.join(tmux_formats),)  # output
 
-        proc = self.cmd('list-sessions', *tmux_args)
+        proc = self.cmd("list-sessions", *tmux_args)
 
         if proc.stderr:
             raise exc.LibTmuxException(proc.stderr)
 
         sformats = formats.SESSION_FORMATS
-        tmux_formats = ['#{%s}' % format for format in sformats]
+        tmux_formats = ["#{%s}" % format for format in sformats]
         sessions = proc.stdout
 
         # combine format keys with values returned from ``tmux list-sessions``
-        sessions = [dict(zip(sformats, session.split('\t'))) for session in sessions]
+        sessions = [
+            dict(zip(sformats, session.split(formats.FORMAT_SEPARATOR)))
+            for session in sessions
+        ]
 
         # clear up empty dict
-        sessions = [
-            dict((k, v) for k, v in session.items() if v) for session in sessions
-        ]
+        sessions = [{k: v for k, v in session.items() if v} for session in sessions]
 
         return sessions
 
     @property
-    def _sessions(self):
+    def _sessions(self) -> t.List[SessionDict]:
         """Property / alias to return :meth:`~._list_sessions`."""
 
         return self._list_sessions()
 
-    def list_sessions(self):
+    def list_sessions(self) -> t.List[Session]:
         """
         Return list of :class:`Session` from the ``tmux(1)`` session.
 
@@ -180,14 +185,14 @@ class Server(TmuxRelationalObject, EnvironmentMixin):
         return [Session(server=self, **s) for s in self._sessions]
 
     @property
-    def sessions(self):
+    def sessions(self) -> t.List[Session]:
         """Property / alias to return :meth:`~.list_sessions`."""
         return self.list_sessions()
 
     #: Alias :attr:`sessions` for :class:`~libtmux.common.TmuxRelationalObject`
     children = sessions
 
-    def _list_windows(self):
+    def _list_windows(self) -> t.List[WindowDict]:
         """
         Return list of windows in :py:obj:`dict` form.
 
@@ -201,13 +206,13 @@ class Server(TmuxRelationalObject, EnvironmentMixin):
         list of dict
         """
 
-        wformats = ['session_name', 'session_id'] + formats.WINDOW_FORMATS
-        tmux_formats = ['#{%s}' % format for format in wformats]
+        wformats = ["session_name", "session_id"] + formats.WINDOW_FORMATS
+        tmux_formats = ["#{%s}" % format for format in wformats]
 
         proc = self.cmd(
-            'list-windows',  # ``tmux list-windows``
-            '-a',
-            '-F%s' % '\t'.join(tmux_formats),  # output
+            "list-windows",  # ``tmux list-windows``
+            "-a",
+            "-F%s" % formats.FORMAT_SEPARATOR.join(tmux_formats),  # output
         )
 
         if proc.stderr:
@@ -215,18 +220,21 @@ class Server(TmuxRelationalObject, EnvironmentMixin):
 
         windows = proc.stdout
 
-        wformats = ['session_name', 'session_id'] + formats.WINDOW_FORMATS
+        wformats = ["session_name", "session_id"] + formats.WINDOW_FORMATS
 
         # combine format keys with values returned from ``tmux list-windows``
-        windows = [dict(zip(wformats, window.split('\t'))) for window in windows]
+        windows = [
+            dict(zip(wformats, window.split(formats.FORMAT_SEPARATOR)))
+            for window in windows
+        ]
 
         # clear up empty dict
-        windows = [dict((k, v) for k, v in window.items() if v) for window in windows]
+        windows = [{k: v for k, v in window.items() if v} for window in windows]
 
         # tmux < 1.8 doesn't have window_id, use window_name
         for w in windows:
-            if 'window_id' not in w:
-                w['window_id'] = w['window_name']
+            if "window_id" not in w:
+                w["window_id"] = w["window_name"]
 
         if self._windows:
             self._windows[:] = []
@@ -235,7 +243,7 @@ class Server(TmuxRelationalObject, EnvironmentMixin):
 
         return self._windows
 
-    def _update_windows(self):
+    def _update_windows(self) -> "Server":
         """
         Update internal window data and return ``self`` for chainability.
 
@@ -246,7 +254,7 @@ class Server(TmuxRelationalObject, EnvironmentMixin):
         self._list_windows()
         return self
 
-    def _list_panes(self):
+    def _list_panes(self) -> t.List[PaneDict]:
         """
         Return list of panes in :py:obj:`dict` form.
 
@@ -261,15 +269,15 @@ class Server(TmuxRelationalObject, EnvironmentMixin):
         """
 
         pformats = [
-            'session_name',
-            'session_id',
-            'window_index',
-            'window_id',
-            'window_name',
+            "session_name",
+            "session_id",
+            "window_index",
+            "window_id",
+            "window_name",
         ] + formats.PANE_FORMATS
-        tmux_formats = ['#{%s}\t' % f for f in pformats]
+        tmux_formats = [("#{%%s}%s" % formats.FORMAT_SEPARATOR) % f for f in pformats]
 
-        proc = self.cmd('list-panes', '-a', '-F%s' % ''.join(tmux_formats))  # output
+        proc = self.cmd("list-panes", "-a", "-F%s" % "".join(tmux_formats))  # output
 
         if proc.stderr:
             raise exc.LibTmuxException(proc.stderr)
@@ -277,21 +285,24 @@ class Server(TmuxRelationalObject, EnvironmentMixin):
         panes = proc.stdout
 
         pformats = [
-            'session_name',
-            'session_id',
-            'window_index',
-            'window_id',
-            'window_name',
+            "session_name",
+            "session_id",
+            "window_index",
+            "window_id",
+            "window_name",
         ] + formats.PANE_FORMATS
 
         # combine format keys with values returned from ``tmux list-panes``
-        panes = [dict(zip(pformats, window.split('\t'))) for window in panes]
+        panes = [
+            dict(zip(pformats, window.split(formats.FORMAT_SEPARATOR)))
+            for window in panes
+        ]
 
         # clear up empty dict
         panes = [
-            dict(
-                (k, v) for k, v in window.items() if v or k == 'pane_current_path'
-            )  # preserve pane_current_path, in case it entered a new process
+            {
+                k: v for k, v in window.items() if v or k == "pane_current_path"
+            }  # preserve pane_current_path, in case it entered a new process
             # where we may not get a cwd from.
             for window in panes
         ]
@@ -303,7 +314,7 @@ class Server(TmuxRelationalObject, EnvironmentMixin):
 
         return self._panes
 
-    def _update_panes(self):
+    def _update_panes(self) -> "Server":
         """
         Update internal pane data and return ``self`` for chainability.
 
@@ -315,7 +326,7 @@ class Server(TmuxRelationalObject, EnvironmentMixin):
         return self
 
     @property
-    def attached_sessions(self):
+    def attached_sessions(self) -> t.Optional[t.List[Session]]:
         """
         Return active :class:`Session` objects.
 
@@ -328,17 +339,17 @@ class Server(TmuxRelationalObject, EnvironmentMixin):
         attached_sessions = list()
 
         for session in sessions:
-            attached = session.get('session_attached')
+            attached = session.get("session_attached")
             # for now session_active is a unicode
-            if attached != '0':
-                logger.debug('session %s attached', session.get('name'))
+            if attached != "0":
+                logger.debug("session %s attached", session.get("name"))
                 attached_sessions.append(session)
             else:
                 continue
 
         return [Session(server=self, **s) for s in attached_sessions] or None
 
-    def has_session(self, target_session, exact=True):
+    def has_session(self, target_session: str, exact: bool = True) -> bool:
         """
         Return True if session exists. ``$ tmux has-session``.
 
@@ -361,10 +372,10 @@ class Server(TmuxRelationalObject, EnvironmentMixin):
         """
         session_check_name(target_session)
 
-        if exact and has_gte_version('2.1'):
-            target_session = '={}'.format(target_session)
+        if exact and has_gte_version("2.1"):
+            target_session = f"={target_session}"
 
-        proc = self.cmd('has-session', '-t%s' % target_session)
+        proc = self.cmd("has-session", "-t%s" % target_session)
 
         if not proc.returncode:
             return True
@@ -373,9 +384,9 @@ class Server(TmuxRelationalObject, EnvironmentMixin):
 
     def kill_server(self):
         """``$ tmux kill-server``."""
-        self.cmd('kill-server')
+        self.cmd("kill-server")
 
-    def kill_session(self, target_session=None):
+    def kill_session(self, target_session=None) -> "Server":
         """
         Kill the tmux session with ``$ tmux kill-session``, return ``self``.
 
@@ -395,7 +406,7 @@ class Server(TmuxRelationalObject, EnvironmentMixin):
         """
         session_check_name(target_session)
 
-        proc = self.cmd('kill-session', '-t%s' % target_session)
+        proc = self.cmd("kill-session", "-t%s" % target_session)
 
         if proc.stderr:
             raise exc.LibTmuxException(proc.stderr)
@@ -417,7 +428,7 @@ class Server(TmuxRelationalObject, EnvironmentMixin):
         """
         session_check_name(target_session)
 
-        proc = self.cmd('switch-client', '-t%s' % target_session)
+        proc = self.cmd("switch-client", "-t%s" % target_session)
 
         if proc.stderr:
             raise exc.LibTmuxException(proc.stderr)
@@ -438,9 +449,9 @@ class Server(TmuxRelationalObject, EnvironmentMixin):
 
         tmux_args = tuple()
         if target_session:
-            tmux_args += ('-t%s' % target_session,)
+            tmux_args += ("-t%s" % target_session,)
 
-        proc = self.cmd('attach-session', *tmux_args)
+        proc = self.cmd("attach-session", *tmux_args)
 
         if proc.stderr:
             raise exc.LibTmuxException(proc.stderr)
@@ -454,8 +465,8 @@ class Server(TmuxRelationalObject, EnvironmentMixin):
         window_name=None,
         window_command=None,
         *args,
-        **kwargs
-    ):
+        **kwargs,
+    ) -> Session:
         """
         Return :class:`Session` from  ``$ tmux new-session``.
 
@@ -508,45 +519,45 @@ class Server(TmuxRelationalObject, EnvironmentMixin):
 
         if self.has_session(session_name):
             if kill_session:
-                self.cmd('kill-session', '-t%s' % session_name)
-                logger.info('session %s exists. killed it.' % session_name)
+                self.cmd("kill-session", "-t%s" % session_name)
+                logger.info("session %s exists. killed it." % session_name)
             else:
-                raise exc.TmuxSessionExists('Session named %s exists' % session_name)
+                raise exc.TmuxSessionExists("Session named %s exists" % session_name)
 
-        logger.debug('creating session %s' % session_name)
+        logger.debug("creating session %s" % session_name)
 
         sformats = formats.SESSION_FORMATS
-        tmux_formats = ['#{%s}' % f for f in sformats]
+        tmux_formats = ["#{%s}" % f for f in sformats]
 
-        env = os.environ.get('TMUX')
+        env = os.environ.get("TMUX")
 
         if env:
-            del os.environ['TMUX']
+            del os.environ["TMUX"]
 
         tmux_args = (
-            '-s%s' % session_name,
-            '-P',
-            '-F%s' % '\t'.join(tmux_formats),  # output
+            "-s%s" % session_name,
+            "-P",
+            "-F%s" % formats.FORMAT_SEPARATOR.join(tmux_formats),  # output
         )
 
         if not attach:
-            tmux_args += ('-d',)
+            tmux_args += ("-d",)
 
         if start_directory:
-            tmux_args += ('-c', start_directory)
+            tmux_args += ("-c", start_directory)
 
         if window_name:
-            tmux_args += ('-n', window_name)
+            tmux_args += ("-n", window_name)
 
         # tmux 2.6 gives unattached sessions a tiny default area
         # no need send in -x/-y if they're in a client already, though
-        if has_gte_version('2.6') and 'TMUX' not in os.environ:
-            tmux_args += ('-x', 800, '-y', 600)
+        if has_gte_version("2.6") and "TMUX" not in os.environ:
+            tmux_args += ("-x", 800, "-y", 600)
 
         if window_command:
             tmux_args += (window_command,)
 
-        proc = self.cmd('new-session', *tmux_args)
+        proc = self.cmd("new-session", *tmux_args)
 
         if proc.stderr:
             raise exc.LibTmuxException(proc.stderr)
@@ -554,13 +565,13 @@ class Server(TmuxRelationalObject, EnvironmentMixin):
         session = proc.stdout[0]
 
         if env:
-            os.environ['TMUX'] = env
+            os.environ["TMUX"] = env
 
         # combine format keys with values returned from ``tmux list-windows``
-        session = dict(zip(sformats, session.split('\t')))
+        session = dict(zip(sformats, session.split(formats.FORMAT_SEPARATOR)))
 
         # clear up empty dict
-        session = dict((k, v) for k, v in session.items() if v)
+        session = {k: v for k, v in session.items() if v}
 
         session = Session(server=self, **session)
 
