@@ -1,8 +1,4 @@
-# -*- coding: utf-8 -*-
 """Helper methods for libtmux and downstream libtmux libraries."""
-
-from __future__ import absolute_import, unicode_literals, with_statement
-
 import contextlib
 import logging
 import os
@@ -11,13 +7,13 @@ import time
 
 logger = logging.getLogger(__name__)
 
-TEST_SESSION_PREFIX = 'libtmux_'
-RETRY_TIMEOUT_SECONDS = int(os.getenv('RETRY_TIMEOUT_SECONDS', 8))
+TEST_SESSION_PREFIX = "libtmux_"
+RETRY_TIMEOUT_SECONDS = int(os.getenv("RETRY_TIMEOUT_SECONDS", 8))
 
 namer = tempfile._RandomNameSequence()
 current_dir = os.path.abspath(os.path.dirname(__file__))
-example_dir = os.path.abspath(os.path.join(current_dir, '..', 'examples'))
-fixtures_dir = os.path.realpath(os.path.join(current_dir, 'fixtures'))
+example_dir = os.path.abspath(os.path.join(current_dir, "..", "examples"))
+fixtures_dir = os.path.realpath(os.path.join(current_dir, "fixtures"))
 
 
 def retry(seconds=RETRY_TIMEOUT_SECONDS):
@@ -39,10 +35,10 @@ def retry(seconds=RETRY_TIMEOUT_SECONDS):
     --------
 
     >>> while retry():
-    ...      p = w.attached_pane
-    ...      p.server._update_panes()
-    ...      if p.current_path == pane_path:
-    ...          break
+    ...     p = w.attached_pane
+    ...     p.server._update_panes()
+    ...     if p.current_path == pane_path:
+    ...         break
     """
     return (lambda: time.time() < time.time() + seconds)()
 
@@ -131,8 +127,8 @@ def temp_session(server, *args, **kwargs):
     ...     session.new_window(window_name='my window')
     """
 
-    if 'session_name' in kwargs:
-        session_name = kwargs.pop('session_name')
+    if "session_name" in kwargs:
+        session_name = kwargs.pop("session_name")
     else:
         session_name = get_test_session_name(server)
 
@@ -180,15 +176,15 @@ def temp_window(session, *args, **kwargs):
     ...     my_pane = window.split_window()
     """
 
-    if 'window_name' not in kwargs:
+    if "window_name" not in kwargs:
         window_name = get_test_window_name(session)
     else:
-        window_name = kwargs.pop('window_name')
+        window_name = kwargs.pop("window_name")
 
     window = session.new_window(window_name, *args, **kwargs)
 
     # Get ``window_id`` before returning it, it may be killed within context.
-    window_id = window.get('window_id')
+    window_id = window.get("window_id")
 
     try:
         yield session
@@ -198,7 +194,7 @@ def temp_window(session, *args, **kwargs):
     return
 
 
-class EnvironmentVarGuard(object):
+class EnvironmentVarGuard:
 
     """Mock environmental variables safetly.
 
